@@ -28,6 +28,8 @@
 #include <language/duchain/duchain.h>
 #include <language/duchain/stringhelpers.h>
 #include <language/duchain/parsingenvironment.h>
+#include <language/duchain/types/unsuretype.h>
+#include <language/duchain/types/integraltype.h>
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
 #include <interfaces/iuicontroller.h>
@@ -42,6 +44,10 @@
 #include "declarations/classdeclaration.h"
 #include "declarations/classmethoddeclaration.h"
 #include "declarations/functiondeclaration.h"
+#include "types/indexedcontainer.h"
+#include "types/integraltypeextended.h"
+#include "expressionparser.h"
+#include "expressionvisitor.h"
 
 #include "duchaindebug.h"
 
@@ -84,11 +90,11 @@ DeclarationPointer findDeclarationImportHelper(DUContext* currentContext, const
         DeclarationType declarationType)
 {
     /// Qualified identifier for 'self'
-    static const QualifiedIdentifier selfQId("self");
+    static const QualifiedIdentifier selfQId(QStringLiteral("self"));
     /// Qualified identifier for 'parent'
-    static const QualifiedIdentifier parentQId("parent");
+    static const QualifiedIdentifier parentQId(QStringLiteral("parent"));
     /// Qualified identifier for 'static'
-    static const QualifiedIdentifier staticQId("static");
+    static const QualifiedIdentifier staticQId(QStringLiteral("static"));
 
     ifDebug(qCDebug(DUCHAIN) << id.toString() << declarationType;)
     if (declarationType == ClassDeclarationType && id == selfQId) {
@@ -112,7 +118,7 @@ DeclarationPointer findDeclarationImportHelper(DUContext* currentContext, const
     } else if (declarationType == ClassDeclarationType && id == parentQId) {
         //there can be just one Class-Context imported
         DUChainReadLocker lock;
-        DUContext* classCtx = 0;
+        DUContext* classCtx = nullptr;
         if (currentContext->type() == DUContext::Class) {
             classCtx = currentContext;
         } else if (currentContext->parentContext() && currentContext->parentContext()->type() == DUContext::Class) {
@@ -181,7 +187,7 @@ DeclarationPointer findDeclarationInPST(DUContext* currentContext, QualifiedIden
 {
     ifDebug(qCDebug(DUCHAIN) << "PST: " << id.toString() << declarationType;)
     uint nr;
-    const IndexedDeclaration* declarations = 0;
+    const IndexedDeclaration* declarations = nullptr;
     DUChainWriteLocker wlock;
     PersistentSymbolTable::self().declarations(id, nr, declarations);
     ifDebug(qCDebug(DUCHAIN) << "found declarations:" << nr;)
@@ -257,12 +263,12 @@ QByteArray formatComment(AstNode* node, EditorIntegrator* editor)
 class ScalarExpressionVisitor : public DefaultVisitor
 {
 public:
-    ScalarExpressionVisitor() : m_node(0) {}
+    ScalarExpressionVisitor() : m_node(nullptr) {}
     CommonScalarAst* node() const {
         return m_node;
     }
 private:
-    virtual void visitCommonScalar(CommonScalarAst* node) {
+    void visitCommonScalar(CommonScalarAst* node) override {
         m_node = node;
     }
     CommonScalarAst* m_node;
@@ -338,7 +344,7 @@ IndexedString getIncludeFileForNode(UnaryExpressionAst* node, EditorIntegrator*
         if (scalar && scalar->string != -1) {
             QString str = editor->parseSession()->symbol(scalar->string);
             str = str.mid(1, str.length() - 2);
-            if ( str == "." || str == ".." || str.endsWith('/') ) {
+            if ( str == QLatin1String(".") || str == QLatin1String("..") || str.endsWith('/') ) {
                 return IndexedString();
             }
             return findIncludeFile(str, editor->parseSession()->currentDocument());
@@ -370,7 +376,7 @@ QString prettyName(Declaration* dec) {
 
 const KDevelop::IndexedString& internalFunctionFile()
 {
-    static const KDevelop::IndexedString internalFile(QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kdevphpsupport/phpfunctions.php"));
+    static const KDevelop::IndexedString internalFile(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kdevphpsupport/phpfunctions.php")));
     return internalFile;
 }
 
@@ -382,7 +388,7 @@ const KDevelop::IndexedString& phpLanguageString()
 
 const IndexedString& internalTestFile()
 {
-    static const KDevelop::IndexedString phpUnitFile(QStandardPaths::locate(QStandardPaths::GenericDataLocation, "kdevphpsupport/phpunitdeclarations.php"));
+    static const KDevelop::IndexedString phpUnitFile(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kdevphpsupport/phpunitdeclarations.php")));
     return phpUnitFile;
 }
 
@@ -418,4 +424,136 @@ QualifiedIdentifier identifierWithNamespace(const QualifiedIdentifier& base, DUC
     }
 }
 
+template <class T>
+AbstractType::Ptr determineTypehint(const T* parameterType, EditorIntegrator *editor, DUContext* currentContext)
+{
+    Q_ASSERT(parameterType);
+    AbstractType::Ptr type;
+
+    if (parameterType->objectType) {
+        //don't use openTypeFromName as it uses cursor for findDeclarations
+        DeclarationPointer decl = findDeclarationImportHelper(currentContext,
+                                                    identifierForNamespace(parameterType->objectType, editor), ClassDeclarationType);
+        if (decl) {
+            type = decl->abstractType();
+        }
+    } else if (parameterType->arrayType != -1) {
+        type = AbstractType::Ptr(new IntegralType(IntegralType::TypeArray));
+    } else if (parameterType->boolType != -1) {
+        type = AbstractType::Ptr(new IntegralType(IntegralType::TypeBoolean));
+    } else if (parameterType->floatType != -1) {
+        type = AbstractType::Ptr(new IntegralType(IntegralType::TypeFloat));
+    } else if (parameterType->intType != -1) {
+        type = AbstractType::Ptr(new IntegralType(IntegralType::TypeInt));
+    } else if (parameterType->stringType != -1) {
+        type = AbstractType::Ptr(new IntegralType(IntegralType::TypeString));
+    } else if (parameterType->callableType != -1) {
+        type = AbstractType::Ptr(new IntegralTypeExtended(IntegralTypeExtended::TypeCallable));
+    } else if (parameterType->iterableType != -1) {
+        DeclarationPointer traversableDecl = findDeclarationImportHelper(currentContext, QualifiedIdentifier("traversable"), ClassDeclarationType);
+
+        if (traversableDecl) {
+            UnsureType::Ptr unsure(new UnsureType());
+            AbstractType::Ptr arrayType = AbstractType::Ptr(new IntegralType(IntegralType::TypeArray));
+            unsure->addType(arrayType->indexed());
+            unsure->addType(traversableDecl->abstractType()->indexed());
+
+            type = AbstractType::Ptr(unsure);
+        }
+    }
+
+    if (type && parameterType->isNullable != -1) {
+        AbstractType::Ptr nullType = AbstractType::Ptr(new IntegralType(IntegralType::TypeNull));
+        if (type.cast<UnsureType>()) {
+            UnsureType::Ptr unsure = type.cast<UnsureType>();
+            unsure->addType(nullType->indexed());
+        } else {
+            UnsureType::Ptr unsure(new UnsureType());
+            unsure->addType(type->indexed());
+            unsure->addType(nullType->indexed());
+
+            type = AbstractType::Ptr(unsure);
+        }
+    }
+
+    return type;
+}
+
+AbstractType::Ptr parameterType(const ParameterAst* node, AbstractType::Ptr phpDocTypehint, EditorIntegrator* editor, DUContext* currentContext)
+{
+    AbstractType::Ptr type;
+    if (node->parameterType) {
+        type = determineTypehint(node->parameterType, editor, currentContext);
+    }
+    if (node->defaultValue) {
+        ExpressionVisitor v(editor);
+        node->defaultValue->ducontext = currentContext;
+        v.visitNode(node->defaultValue);
+        AbstractType::Ptr defaultValueType = v.result().type();
+
+        /*
+         * If a typehint is already set, default values can only be the same type or `null` in PHP
+         * If it's the same type, the type is already correctly set,
+         *    so we can ignore this case.
+         * If it's null (which cannot be a typehint), add it as UnsureType
+         */
+        if (type && defaultValueType.cast<IntegralType>() && defaultValueType.cast<IntegralType>()->dataType() == IntegralType::TypeNull) {
+            if (type.cast<UnsureType>()) {
+                UnsureType::Ptr unsure = type.cast<UnsureType>();
+                AbstractType::Ptr nullType = AbstractType::Ptr(new IntegralType(IntegralType::TypeNull));
+                unsure->addType(defaultValueType->indexed());
+            } else {
+                UnsureType::Ptr unsure(new UnsureType());
+                unsure->addType(type->indexed());
+                unsure->addType(defaultValueType->indexed());
+
+                type = AbstractType::Ptr(unsure);
+            }
+        } else {
+            //Otherwise, let the default value dictate the parameter type
+            type = defaultValueType;
+        }
+    }
+    if (!type) {
+        if (phpDocTypehint) {
+            type = phpDocTypehint;
+        } else {
+            type = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
+        }
+    }
+
+    if ( node->isRef != -1 ) {
+      ReferenceType::Ptr p( new ReferenceType() );
+      p->setBaseType( type );
+
+      type = p.cast<AbstractType>();
+    }
+
+    if (node->isVariadic != -1) {
+        auto *container = new IndexedContainer();
+        const IndexedString *containerType = new IndexedString("array");
+        container->addEntry(type);
+        container->setPrettyName(*containerType);
+        type = AbstractType::Ptr(container);
+    }
+
+    Q_ASSERT(type);
+    return type;
+}
+
+AbstractType::Ptr returnType(const ReturnTypeAst* node, AbstractType::Ptr phpDocTypehint, EditorIntegrator* editor, DUContext* currentContext) {
+    AbstractType::Ptr type;
+    if (node) {
+        if (node->voidType != -1) {
+            type = AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid));
+        } else {
+            type = determineTypehint(node, editor, currentContext);
+        }
+    }
+    if (!type) {
+        type = phpDocTypehint;
+    }
+    return type;
+}
+
 }
